from HtmlCreator.Body.Input.Input import Input


# Single
class IpBoxS(Input):

    def _getType(self):
        return 'radio'


# Multiple
class IpBoxM(Input):

    def _getType(self):
        return 'checkbox'



from abc import ABC, abstractmethod

from HtmlCreator import EmptyElement


class Input(EmptyElement, ABC):
    def __init__(self):
        self.__name: str = None
        self.__value: str = None

    def _getEleKey(self):
        return 'input'

    @abstractmethod
    def _getType(self):
        return ''

    def _getAttrs(self) -> dict:
        rt = super(Input, self)._getAttrs()
        rt['type'] = self._getType()
        if self.__name is not None:
            rt['name'] = self.__name
        return rt


    def name(self, name: str):
        self.__name = name
        return self


    def value(self, value: str):
        self.__value = value
        return self



from HtmlCreator.EmptyElement import EmptyElement


class Meta(EmptyElement):
    def _getEleKey(self):
        return 'meta'


class MCharset(Meta):
    UTF8 = 'utf-8'

    def __init__(self, charset: str = 'utf-8'):
        self.__charset = charset

    def _getAttrs(self):
        return {
            'charset': self.__charset
        }


class MKeywords(Meta):
    def __init__(self, keywords: list):
        # 列表中放搜索引擎关键词
        self.__keywords = keywords


class MDescription(Meta):
    def __init__(self, content: str):
        # 放网站描述
        self.__content = content


class MAuthor(Meta):
    def __init__(self, name: str):
        self.__name = name


class MAutoRefresh(Meta):
    # <meta http-equiv="refresh" content="30">
    def __init__(self, seconds: int = 30):
        self.__seconds = seconds



from HtmlCreator import EmptyElement


class Image(EmptyElement):
    def _getEleKey(self):
        return 'img'

    __width = None
    __height = None

    def __init__(self,
                 src: str,
                 alt: str = 'can not load image',
                 is_lazy: bool = True):
        # src is a image source
        self.__src = src
        self.__alt = alt
        self.__is_lazy = is_lazy

    def setSize(self, width: int, height: int):
        self.__width = width
        self.__height = height

    def _getAttrs(self) -> dict:
        rt = {
            'src': self.__src,
            'alt': self.__alt,
        }
        if self.__width is not None and self.__height is not None:
            rt['width'] = self.__width
            rt['height'] = self.__height
        if self.__is_lazy:
            rt['loading'] = 'lazy'
        return rt



from HtmlCreator.ExistElement import ExistElement
from HtmlCreator.Head.Title import Title
from HtmlCreator.Head.Meta import MCharset


class Head(ExistElement):
    def _getEleKey(self):
        return 'head'

    def __init__(self, title: str):
        super().__init__()
        self.title = title
        self.addElement(MCharset())
        self.addElement(Title(title))



from HtmlCreator import Element


class Text(Element):
    def _getEleKey(self):
        return ''

    def __str__(self):
        return self.text

    def __init__(self, text: str):
        text = self.replace(text)
        self.text = text

    @staticmethod
    def replace(text: str) -> str:
        text = text.replace('&', '&amp;')
        text = text.replace('<', '&lt;')
        text = text.replace('>', '&gt;')
        text = text.replace('"', '&quot;')
        text = text.replace("'", '&#39;')
        text = text.replace(" ", '&nbsp;')
        return text





from HtmlCreator import ExistElement

class Table(ExistElement):
    def __init__(self,border:int=1,caption:str=None,headings:list = []):
        self.__border = border
        self.__caption = caption
        self.__headings = headings

    def _getAttrs(self):
        return {
            'border':self.__border,
        }

class TableElement(ExistElement):
    ...

class TCaption(TableElement):
    ...

class Tr(TableElement):
    ...

class Th(TableElement):
    ...

class Td(TableElement):
    ...


from abc import ABCMeta

from typing import List, Optional, Union

from HtmlCreator import Element
from HtmlCreator.Body.Text import Text


class ExistElement(Element, metaclass=ABCMeta):
    # abstract class
    def __init__(self, content: Optional[Union[List[Element]]] = None, ):
        if content is None:
            self.elements = []
        else:
            self.elements = content

    def text(self, text:str):
        self.elements.append(Text(text))
        return self

    def addElement(self, ele: Element):
        self.elements.append(ele)
        return self

    def getElements(self):
        return self.elements

    def _getContStr(self):
        rt = ''
        for ele in self.getElements():
            rt += str(ele)
        return rt

    def __str__(self):
        return self._wrapContent()

    def _wrapContent(self):
        return f'<{self._getEleKey()}{self._getAttrStr()}>{self._getContStr()}</{self._getEleKey()}>'



from abc import ABCMeta, abstractmethod


class Element(metaclass=ABCMeta):
    
    @abstractmethod
    def _getEleKey(self):
        ...

    @abstractmethod
    def __str__(self):
        ...

    def _getAttrs(self) -> dict:
        return {}

    def _getAttrStr(self) -> str:
        attrs = self._getAttrs()
        if len(attrs) > 0:
            rt = ''
            for k in attrs.keys():
                rt += f' {k}="{attrs[k]}"'
            return rt
        return ''



from HtmlCreator.ExistElement import ExistElement


class Title(ExistElement):

    def _getEleKey(self):
        return 'title'

    def __init__(self, title: str):
        super().__init__()
        self.__title = title

    def _getContStr(self):
        return self.__title



from HtmlCreator.Body.Input.Input import Input


class IpButton(Input):
    TYPE_BUTTON = 'button'
    TYPE_SUBMIT = 'submit'
    TYPE_RESET = 'reset'


    def __init__(self,type:str = 'button'):
        self.__type = type

    def _getType(self):
        return self.__type


from HtmlCreator.ExistElement import ExistElement
from HtmlCreator.Head.Head import Head
from HtmlCreator.Body.Body import Body


class Html(ExistElement):
    def __init__(self, title: str):
        super().__init__()
        self.addElement(Head(title))
        self.body = Body()
        self.addElement(self.body)

    def _getEleKey(self):
        return 'html'

    def __str__(self):
        return '<!DOCTYPE html>' + super().__str__()


#!/usr/bin/env python
# coding: utf-8

from setuptools import setup

long_description = '''
# readme:

this is a package to create html with python language
though it will be used by myself, also you can use it.
'''


def get_readme():
    with open('README.md', 'r') as f:
        return ''.join(f.readlines())


setup(
    name='html_builder',
    version='0.0.3',
    author='normidar',
    author_email='normidar7@gmail.com',
    url='https://www.normidar.com',
    description=get_readme(),
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=[
        'HtmlCreator',
        'HtmlCreator.Body',
        'HtmlCreator.Head',
        'HtmlCreator.Body.Input'
    ],
    install_requires=[],
    entry_points={
        'console_scripts': []
    }
)



from HtmlCreator.Element import Element
from HtmlCreator.ExistElement import ExistElement
from HtmlCreator.EmptyElement import EmptyElement
from HtmlCreator.Html import Html

from HtmlCreator.Body.Body import Body





from typing import Optional

from HtmlCreator import *
from HtmlCreator.Body.Text import Text


class Heading(ExistElement):
    def __init__(self, size: int):
        # 传入1至6的数来创建大小
        super().__init__()

        if size < 1 or size > 6:
            raise ValueError("Heading over 1~6")
        self.__size = size

    def _getEleKey(self):
        return f'h{self.__size}'



from HtmlCreator import *


class Anchor(ExistElement):
    def __init__(self, url: str, to_url=False):
        super().__init__()
        if to_url:
            if url[:8] != 'https://':
                if url[:7] != 'http://':
                    url = 'https://' + url
        self.__href = url

    @classmethod
    def linkToId(cls, id: str):
        super().__init__()
        cls.__href = f'#{id}'

    def _getAttrs(self):
        return {
            'href': self.__href
        }

    def _getEleKey(self):
        return 'a'
